#include "sdl/SDL.h"
#include "sdl/SDL_rotozoom.h"
#include "sdl/SDL_framerate.h"
#include "sdl/SDL_image.h"
#include "lua51/lua.hpp"
#include <string>
#include "globals.h"

static  std::string tick_func;
static SDL_TimerID timer = 0;
static FPSmanager *manager = new FPSmanager;

//function for loading the configuration data
//returns the screen
SDL_Surface *load_config(lua_State *L);

//the function that is run when the timer ticks
//param ends up being the lua state.
Uint32 timer_tick(Uint32 interval, void *param);


//function for setting a timer from lua
static int l_set_timer(lua_State *L){
  Uint32 interval = (Uint32)luaL_checknumber(L, 1);
  tick_func = std::string(luaL_checkstring(L,2));
  
  //create timer if none exists, else
  //delete current and create a new one
  if(timer == 0){
    timer = SDL_AddTimer(interval, timer_tick, L);
  }else{
    SDL_RemoveTimer(timer);
    timer = SDL_AddTimer(interval, timer_tick, L);      
  }
  
  return 0; //return no values
}

//function for removing a timer from lua
static int l_unset_timer(lua_State *L){
  //create timer if none exists, else
  //delete current and create a new one
  if(timer == 0){
    return 0;
  }else{
    SDL_RemoveTimer(timer);      
  }
  
  return 0; //return no values
}

//function for loading a surface -- deprecated
static int l_loadbmp(lua_State *L){
  SDL_Surface *tmp = SDL_LoadBMP(luaL_checkstring(L,1));
  lua_pushlightuserdata(L, tmp);
  lua_pushinteger(L, (lua_Integer)tmp->w);
  lua_pushinteger(L, (lua_Integer)tmp->h);
  return 3;
}

//function for loading a surface (transp) -- deprecated
static int l_loadbmp_t(lua_State *L){
  SDL_Surface *tmp = SDL_LoadBMP(luaL_checkstring(L,1));
  SDL_SetColorKey(tmp, SDL_SRCCOLORKEY, SDL_MapRGB(tmp->format, 255, 0, 255));
  lua_pushlightuserdata(L, tmp);
  lua_pushinteger(L, (lua_Integer)tmp->w);
  lua_pushinteger(L, (lua_Integer)tmp->h);
  return 3;     
}

//function for loading a surface from many different formats (except TIFF)
static int l_loadimg(lua_State *L){
  SDL_Surface *tmp = IMG_Load(luaL_checkstring(L,1));
  lua_pushlightuserdata(L, tmp);
  lua_pushinteger(L, (lua_Integer)tmp->w);
  lua_pushinteger(L, (lua_Integer)tmp->h);
  return 3;
}

//function for updating the display surface
static int l_flip(lua_State *L){
  SDL_Flip(screen);
  return 0;
}

//function for simple blits (non-animated)
static int l_blit(lua_State *L){
  SDL_Surface *tmp = (SDL_Surface*)lua_touserdata(L,1);
  int x = lua_tointeger(L,2);
  int y = lua_tointeger(L,3);
  
  SDL_Rect *dest = new SDL_Rect;
  dest->x = x;
  dest->y = y;
  dest->w = tmp->w;
  dest->h = tmp->h;
  
  SDL_BlitSurface(tmp, &tmp->clip_rect, screen, dest);
  delete dest;
  return 0;      
}

//function for blitting a frame of an animation
//frames are laid out horizontally
//frames start at 0
static int l_blitframe(lua_State *L){
  SDL_Surface *tmp = (SDL_Surface*)lua_touserdata(L,1);
  int x = lua_tointeger(L,2);
  int y = lua_tointeger(L,3);
  int numframes = lua_tointeger(L,4);
  int frame = lua_tointeger(L,5);
  
  //make the dest rect
  SDL_Rect *dest = new SDL_Rect;
  dest->x = x;
  dest->y = y;
  dest->w = tmp->w/numframes;
  dest->h = tmp->h;
  
  //make the src rect based on the frame info
  SDL_Rect *src = new SDL_Rect;
  src->x = frame * (tmp->w/numframes);
  src->y = tmp->clip_rect.y;
  src->w = tmp->w/numframes;
  src->h = tmp->h;
  
  SDL_BlitSurface(tmp, src, screen, dest);
  delete dest;
  delete src;
  return 0;      
}

//delay function... use this instead of timers? definitely
static int l_delay(lua_State *L){
  int time = (int)lua_tonumber(L, 1);
  SDL_Delay(time);
  return 0;      
}

//get mouse position
static int l_mouse_state(lua_State *L){
  int x=0; int y=0; bool left=false; bool right=false;
  
  SDL_PumpEvents();
  Uint8 mouse_state = SDL_GetMouseState(&x, &y);
  if(mouse_state&SDL_BUTTON(1))
    left = true;
  if(mouse_state&SDL_BUTTON(3))
    right = true;
  
  lua_pushinteger(L, (lua_Integer)x);
  lua_pushinteger(L, (lua_Integer)y);
  lua_pushboolean(L, left);
  lua_pushboolean(L, right);
  
  return 4;       
}

//cursor toggler
static int l_show_cursor(lua_State *L){
  bool show = lua_toboolean(L, 1);
  
  SDL_ShowCursor((show ? SDL_ENABLE : SDL_DISABLE));
       
  return 0;       
}

//fills screen with a color
static int l_fill_screen(lua_State *L){
  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, lua_tointeger(L,1),lua_tointeger(L,1),lua_tointeger(L,1)));
  return 0;      
}

//free surface interface (sad but true (-_-))
static int l_delete_image(lua_State *L){
  SDL_Surface *tmp = (SDL_Surface*)lua_touserdata(L,1);
  SDL_FreeSurface(tmp);
  lua_pushnil(L);
  return 1;      
}


//rotozoomer interface
static int l_rotzoom(lua_State *L){
  SDL_Surface *tmp = (SDL_Surface*)lua_touserdata(L,1);
  double angle = lua_tonumber(L,2);
  double zoom = lua_tonumber(L,3);
  
  SDL_Surface *zoomed = rotozoomSurface(tmp, angle, zoom, SMOOTHING_OFF);
  
  lua_pushlightuserdata(L, zoomed);
  lua_pushinteger(L, (lua_Integer)zoomed->w);
  lua_pushinteger(L, (lua_Integer)zoomed->h);
  return 3;      
}

//framerate functions
//sophisticated functions that generate proper delays
//to ensure a (relatively) consistent framerate
static int l_initfps(lua_State *L){
  SDL_initFramerate(manager);
  return 0;
}

//sets fps
static int l_setfps(lua_State *L){
  int fps = (int)lua_tonumber(L, 1);
  if(!manager){
    SDL_initFramerate(manager);
  }
  SDL_setFramerate(manager, fps); 
  return 0;
}

//returns the fps
static int l_getfps(lua_State *L){
  lua_pushinteger(L, SDL_getFramerate(manager));
  return 1;       
}

//creates a delay
static int l_delayfps(lua_State *L){
  SDL_framerateDelay(manager);
  return 0;       
}
